#include <ESP8266WiFi.h>
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"
//#include "mbed.h"

// WiFi parameters
#define WLAN_SSID "********"     // Insert here your WiFi network name
#define WLAN_PASS "********"    // Insert here the password for your WiFi network

#define AIO_SERVER      "io.adafruit.com"
#define AIO_SERVERPORT  1883
#define AIO_USERNAME    "*******"             // Insert here your AIO username
#define AIO_KEY         "*******"            // Insert here your AIO key

#define TRIGGER D6
#define ECHO    D5

#define AUD A0

#define SPEAKER D7

// defines variables for calc_distance function
long duration;
int distance_data;

int standart_distance;
float sampleArr[8000];

// Functions declarations
void connect();
//void calc_distance();
WiFiClient client;

// MQTT definitions
const char MQTT_SERVER[] = AIO_SERVER;
const char MQTT_CLIENTID[] = AIO_KEY __DATE__ __TIME__;
const char MQTT_USERNAME[] = AIO_USERNAME;
const char MQTT_PASSWORD[] = AIO_KEY;

// Adafruit IO client definitions
Adafruit_MQTT_Client mqtt(&client, MQTT_SERVER, AIO_SERVERPORT, MQTT_CLIENTID, MQTT_USERNAME, MQTT_PASSWORD);

// Adafruit IO Feeds definition
const char DISTANCE_FEED[] = AIO_USERNAME "/feeds/Distance";
const char SOUND_FEED[] = AIO_USERNAME "/feeds/Sound";
const char ALERTS_FEED[] = AIO_USERNAME "/feeds/Alerts";


Adafruit_MQTT_Publish distance = Adafruit_MQTT_Publish(&mqtt, DISTANCE_FEED);
Adafruit_MQTT_Publish sound = Adafruit_MQTT_Publish(&mqtt, SOUND_FEED);
Adafruit_MQTT_Publish alerts = Adafruit_MQTT_Publish(&mqtt, ALERTS_FEED);


void setup() {
  // put your setup code here, to run once:
  Serial.begin (9600);
  pinMode(TRIGGER, OUTPUT);
  pinMode(ECHO, INPUT);
  pinMode(AUD, OUTPUT);
//  pinMode(SPEAKER, OUTPUT);

  standart_distance = calc_distance();
  Serial.print("STANDART DISTANCE IS:  ");
  Serial.println(standart_distance);
  init_wifi_connection();
  

  // Connect to MQTT and Adafruit IO
  connect();
}

void loop() {
  // ping Adafruit IO a few times to make sure we remain connected
  if (!mqtt.ping(3)) {
    if (!mqtt.connected()) connect();     // reconnect to adafruit io
  }
  distance_data = calc_distance();
  publish_distance(distance_data, "distance");
  if(check_dist(distance_data)){
    alarm_on();
  }
  delay(10000);
}


void alarm_on(){
  for(int i = 0; i < 100; i++){
    publish_sound(analogRead(AUD), "sound");
    tone(SPEAKER, 2000, 500);
    delay(1000);
    tone(SPEAKER, 2000, 500);
    
  }
}

bool check_dist(int d){
  if(d < 0.9 * standart_distance){
    return true;
  }
  return false;
}


void record() {
    float s;
    //record sound for 1 second
    for(int i=0; i<10; i++) {
        s = analogRead(AUD);
        sampleArr[i] = s; //put samples in array
        delay(0.000125f); //sample rate of 8000 Hz
    }
}
 
void publish_distance(int d, String s){
  if(!distance.publish(d)){
    Serial.print("Failed to publish distance: ");
    Serial.println(s);
  } else {
    Serial.print(s);
    Serial.print(": ");
    Serial.print(d);
    Serial.println(" published to the feed!");
  }
}

void publish_sound(int d, String s){
  if(!sound.publish(d)){
    Serial.print("Failed to publish sound: ");
    Serial.println(s);
  } else {
    Serial.print(s);
    Serial.print(": ");
    Serial.print(d);
    Serial.println(" published to the feed!");
    delay(1000);
  }
}

int calc_distance(){
  int d;
  // Clears the trigPin
  digitalWrite(TRIGGER, LOW);
  delayMicroseconds(2);
   
  // Sets the trigPin on HIGH state for 10 micro seconds
  digitalWrite(TRIGGER, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIGGER, LOW);
   
  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(ECHO, HIGH);
   
  // Calculating the distance
  d= duration*0.034/2;

  return d;
}


/**
 * This method connects to the Adafruit IO and MQTT 
 **/
void init_wifi_connection() {
  Serial.print("Connecting to ");
  Serial.println(WLAN_SSID);
  WiFi.begin(WLAN_SSID, WLAN_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    delay(10);
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}


/**
 * This method connects to the Adafruit IO and MQTT 
 **/
void connect() {
  Serial.print("Connecting to Adafruit IO... ");
  int8_t ret;
  while ((ret = mqtt.connect()) != 0) {
    switch (ret) {
      case 1: Serial.println("Wrong protocol"); break;
      case 2: Serial.println("ID rejected"); break;
      case 3: Serial.println("Server unavail"); break;
      case 4: Serial.println("Bad user/pass"); break;
      case 5: Serial.println("Not authed"); break;
      case 6: Serial.println("Failed to subscribe"); break;
      default: Serial.println("Connection failed"); break;
    }
    if (ret >= 0) {
      mqtt.disconnect();
    }
    Serial.println("Retrying connection...");
    delay(10000);
  }
  Serial.println("Adafruit IO Connected!");
}
